/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
#include "ircdserv.h"

void nep(char i)
 {
  context;
  logit("LOAD_CONFIG: Not enought parameters/bad parameter on line %i.\n");
  switch(i)
   {
    case 'A':
     {
      logit("LOAD_CONFIG: Should be A:info1:info2:info3\n");
      break;
     }
    case 'C':
     {
      logit("LOAD_CONFIG: Should be C:hostname:password:servername:port:wildcard number\n");
      break;
     }
    case 'J':
     {
      logit("LOAD_CONFIG: Should be J:servername:description\n");
      break;
     }
    case 'M':
     {
      logit("LOAD_CONFIG: Should be M:servicesname:description:fork:logtofile\n");
      break;
     }
    case 'N':
     {
      logit("LOAD_CONFIG: Should be N:nickname:user:host:ircop(0/1):invisible(0/1):...\n");
      logit("                         ...flag(1=services, 2=juped):realname\n");
      break;
     }
    case 'O':
     {
      logit("LOAD_CONFIG: Should be O:hostmask:password:username:permissions:#chan1,...\n");
      break;
     }
   }
  fatal(1);
 }

void load_config(void)
 {
  FILE *fconfig;
  char c[1024], *d;
  TConn *conn;
  TOper *oper;
  TChanList *chan;
  int i, line;
  int mline, aline, nline;
  char *nick, *user, *host, *ircop, *invisible, *internal, *name;
  
  context;
// opening configuration file
  sprintf(c, "%s/%s", DPATH, CPATH);
  fconfig=fopen(c,"r");
  if(!fconfig)
   {
    logit("LOAD_CONFIG: Config file %s not found.\n", c);
    fatal(1);
   }
  
  context;
// removing old connections !!!!! should be reprogrammed
  while((conn=config.conns))
   {
    config.conns=config.conns->next;
    free(conn);
   }

  context;
// removing old operators !!!!! should be reprogrammed
  while((oper=config.opers))
   {
    while((chan=oper->chanlist))
     {
      oper->chanlist=oper->chanlist->next;
      free(chan);
     }
    config.opers=config.opers->next;
    free(oper);
   }

  context;
// loading input
  nline=mline=aline=line=0;
  logit("LOAD_CONFIG: Loading %s...\n", c);
  while(fgets(c, sizeof(c), fconfig))
   {
    if(c[strlen(c)-1]=='\n') c[strlen(c)-1]='\0';
    line++;
    if(c[0]=='#') continue; // commented line
    if(c[1]!=':') continue; // wrong line
    switch(c[0])
     {
      case 'A':
       {
        context;
        if(aline)
          logit("LOAD_CONFIG: A-line already defined, skipping.\n"); else
         {
          if(!strtok(c, ":")) nep(c[0]);
          for(i=0; i!=3; i++)
            if((d=strtok(NULL, ":"))) strcpy(config.admin[i], d); else nep(c[0]);
          aline=1;
         }
        break;
       }
      case 'C':
       {
        context;
        conn=malloc(sizeof(TConn));
        conn->next=config.conns;
        config.conns=conn;

        if(!strtok(c, ":")) nep(c[0]);
        if((d=strtok(NULL, ":"))) strcpy(conn->host, d); else nep(c[0]);
        if((d=strtok(NULL, ":"))) strcpy(conn->pass, d); else nep(c[0]);
        if((d=strtok(NULL, ":"))) strcpy(conn->name, d); else nep(c[0]);
        if((d=strtok(NULL, ":"))) conn->port=atoi(d); else nep(c[0]);
        if((d=strtok(NULL, ":"))) conn->wildcarding=atoi(d); else nep(c[0]);
        break;
       }
      case 'J':
       {
        context;
        if(!mline)
         {
          logit("LOAD_CONFIG: Error: J-line before/without M-line.\n");
          fatal(1);
         }
        if(!strtok(c, ":")) nep(c[0]);
        if(!(host=strtok(NULL, ":"))) nep(c[0]);
        if(!(name=strtok(NULL, ":"))) nep(c[0]);
        addserver(host, name, 1, 2, config.name);
       }
      case 'M':
       {
        context;
        if(mline)
          logit("LOAD_CONFIG: M-line already defined, skipping.\n"); else
         {
          if(!strtok(c, ":")) nep(c[0]);
          if((d=strtok(NULL, ":"))) strcpy(config.name, d); else nep(c[0]);
          if((d=strtok(NULL, ":"))) strcpy(config.desc, d); else nep(c[0]);
          if((d=strtok(NULL, ":"))) config.fork=atoi(d); else nep(c[0]);
          if((d=strtok(NULL, ":"))) config.uselog=atoi(d); else nep(c[0]);
          addserver(config.name, config.desc, 0, 1, "none");
          mline=1;
         }
        break;
       }
      case 'N':
       {
        context;
        if(!mline)
         {
          logit("LOAD_CONFIG: Error: N-line before/without M-line.\n");
          fatal(1);
         }
        if(!strtok(c, ":")) nep(c[0]);
        if(!(nick=strtok(NULL, ":"))) nep(c[0]);
        if(!(user=strtok(NULL, ":"))) nep(c[0]);
        if(!(host=strtok(NULL, ":"))) nep(c[0]);
        if(!(ircop=strtok(NULL, ":"))) nep(c[0]);
        if(!(invisible=strtok(NULL, ":"))) nep(c[0]);
        if(!(internal=strtok(NULL, ":"))) nep(c[0]);
        if(!(name=strtok(NULL, ":"))) nep(c[0]);
        if(atoi(internal)==1)
         {
          if(nline)
           {
            logit("LOAD_CONFIG: Services nick already defined, skipping.\n");
            continue;
           } else
           {
            nline=1;
            strcpy(config.nick, nick);
           }
         }
        if((atoi(internal)<=0) || (atoi(internal)>=3))
          logit("LOAD_CONFIG: Invalid flag range, skipping...\n");
         else
          adduser(nick, user, host, config.name, 1, 0, atoi(ircop), atoi(invisible), atoi(internal), name);
        break;
       }
      case 'O':
       {
        context;
        oper=malloc(sizeof(TOper));
        oper->next=config.opers;
        config.opers=oper;
        oper->lognick=NULL;
        if(!strtok(c, ":")) nep(c[0]);
        if((d=strtok(NULL, ":"))) strcpy(oper->host, d); else nep(c[0]);
        if((d=strtok(NULL, ":"))) strcpy(oper->pass, d); else nep(c[0]);
        if((d=strtok(NULL, ":"))) strcpy(oper->nick, d); else nep(c[0]);
        if(!(d=strtok(NULL, ":"))) nep(c[0]);
        for(i=0;i!=PERMNO;i++) oper->perms[i]=0;
        for(i=0;d[i];i++) oper->perms[d[i]-65]=1;
        if((d=strtok(NULL, ":"))) strcpy(c, d); else nep(c[0]);
        chan=malloc(sizeof(TChanList));
        oper->chanlist=chan;
        if((d=strtok(c, ","))) strcpy(chan->channel, d); else nep(c[0]);
        while((d=strtok(NULL, ",")))
         {
          chan->next=malloc(sizeof(TChanList));
          chan=chan->next;
          strcpy(chan->channel, d);
         }
        chan->next=NULL;
        break;
       }
     }
   }
  if(!nline || !mline || !aline)
   {
    context;
    logit("LOAD_CONFIG: You haven't A-line or M-line or defined Services nick.\n");
    fatal(1);
   }
  logit("LOAD_CONFIG: Successfully loaded %d lines...\n", line);
  context;
  fclose(fconfig);
 }
