/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
#include "ircdserv.h"

void handle_SIGNAL(int signal)
 {
  switch(signal)
   {
    case SIGINT:
    case SIGQUIT:
    case SIGTERM:
     {
      rec_TERM=1;
      break;
     }
    case SIGHUP:
     {
      rec_HUP=1;
      break;
     }
    case SIGSEGV:
      if(segv==0)
       {
        segv=1;
        logit("Crashed :-(. Check: %s:%d\n", context_file, context_line);
       } else exit(1);
    break;
   }
 }

int main()
 {
  TConn *connhost;
  int i, buflen, err;
  char buffer[8192], *cmd;

  info.servicestart=time(NULL);
  context;
  segv=0;

// config initialization
  config.name[0]=0;
  config.desc[0]=0;
  config.admin[0][0]=0;
  config.admin[1][0]=0;
  config.admin[2][0]=0;
  config.fork=0;
  config.uselog=0;
  config.conns=NULL;
  config.opers=NULL;

// clients initialization
  users=malloc(sizeof(THashUser));
  for(i=0; i!=ESTUSERS; i++) users->hashtable[i]=NULL;
  users->users=0;
  users->maxusers=0;
  users->lusers=0;
  users->avgusers=0;
  users->avglusers=0;
  users->maxlusers=0;
  users->maxusersTS=time(NULL);
  users->maxlusersTS=time(NULL);
  users->avgusersTS=time(NULL);
  users->avglusersTS=time(NULL);
  users->invisible=0;
  users->ircop=0;

// server initialization
  servers=NULL;

  context;

// userhosts initialization
  userhosts=malloc(sizeof(THashUserHost));
  for(i=0; i!=ESTUSERS; i++) userhosts->hashtable[i]=NULL;
  userhosts->userhosts=0;
  userhosts->maxuserhosts=0;
  userhosts->avguserhosts=0;
  userhosts->maxuserhostsTS=time(NULL);
  userhosts->avguserhostsTS=time(NULL);

// hosts initialization
  hosts=malloc(sizeof(THashHost));
  for(i=0; i!=ESTUSERS; i++) hosts->hashtable[i]=NULL;
  hosts->hosts=0;
  hosts->maxhosts=0;
  hosts->avghosts=0;
  hosts->maxhostsTS=time(NULL);
  hosts->avghostsTS=time(NULL);
  
// channels initialization
  channels=malloc(sizeof(THashChannel));
  for(i=0; i!=ESTCHANNELS; i++) channels->hashtable[i]=NULL;
  channels->channels=0;
  channels->maxchannels=0;
  channels->avgchannels=0;
  channels->maxchannelsTS=time(NULL);
  channels->avgchannelsTS=time(NULL);

// statistics initialization
  info.servers=0;
  info.maxservers=0;
  info.avgservers=0;
  info.maxserversTS=time(NULL);
  info.avgserversTS=time(NULL);
  info.kills=0;
  info.joins=0;
  info.nickchngs=0;
  info.servjupes=0;
  info.nickjupes=0;
  info.clones=0;
  info.linktime=0;      // NOT LINKED
  info.linkserver=NULL; // NOT LINKED
  
// fuckover list
  folist=NULL;

// signal handling
  signal(SIGINT, handle_SIGNAL);
  signal(SIGQUIT, handle_SIGNAL);
  signal(SIGTERM, handle_SIGNAL);
  signal(SIGHUP, handle_SIGNAL);
  signal(SIGSEGV, handle_SIGNAL);

// loading configuration file
  context;
  load_config();
  connhost=config.conns;
  while(connectto(connhost))
   {
    connhost=connhost->next;
    if(!connhost) connhost=config.conns;
    sleep(1);
   }
  context;
  while(1) // never-ending loop :-)
   {
    buflen=0;
    buffer[0]=0;
    err=1;
    while((!strchr(buffer,10)) && (err>=0))
     {
      if(rec_TERM)
       {
        wallop("Halting services...(got SIGTERM signal)");
        sleep(1);
        close_sock(upFD);
        exit(0);
       }
      context;
      if(rec_HUP)
       {
        wallop("Reloading conf. file...\n");
        logit("Reloading conf. file...\n");
        load_config();
        rec_HUP=0;
       }
      err=secure_read(upFD, &buffer[buflen], 1);
      buflen+=err;
      buffer[buflen]=0;
      context;
     }
    if(err<0 || disconnect)
     {
      close_sock(upFD);
      delserver(servers->next);
      if(!disconnect)
       {
        if(err==-2) logit("Peer died.\n"); else
                    logit("Connect error.\n");
       }
      logit("Sleeping 5 seconds...\n");
      sleep(5);
      context;
      connhost=connhost->next;
      if(!connhost) connhost=config.conns;
      while(connectto(connhost))
       {
        connhost=connhost->next;
        if(!connhost) connhost=config.conns;
       }
      connectto(connhost);
     } else
     {
      buflen=strchr(buffer,10)-buffer;
      if(buffer[buflen-1]==13) buflen--;
      context;
      buffer[buflen]=0;
      printf("INPUT: %s\n", buffer);
      if(buffer[0]==':')
       {
        strcpy(buffer, buffer+1);
        who=strtok(buffer, " ");
        cmd=strtok(NULL, " ");
       } else
       {
        who=NULL;
        cmd=strtok(buffer, " ");
       }
      whom=strtok(NULL, " ");
      if(whom)
       {
        if(whom[0]==':') whom++;
        args=0;
        while((arg[args]=strtok(NULL, " "))) args++; arg[args]=NULL; args--;
        if(arg[0]) if(arg[0][0]==':') arg[0]++;
       } else arg[0]=NULL;
      context;
      hh_parse(cmd);
     }
   }
  return 0;
 }
