/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
#ifndef _IRCDSERV_H
#define _IRCDSERV_H

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <pwd.h>
#include <crypt.h>
#include <ctype.h>
#include "ss_sock.h"
#include "config.h"
#include <sys/types.h>
#include <sys/wait.h>
#ifndef NO_PTHREAD
#include <pthread.h>
#endif
#ifdef WIN32
#include <windows.h>
#endif

int context_line;
char context_file[256];
FILE *logfile;

#define NO_PTHREAD	1 // will be in configure script
// pthread should be fixed... anybody can help me ?
// fuckover doesn't need another process/thread.. i am right ? -WSX

#define VERSION		"1b11(20Ap2000)"

#define PERMNO		8       // number of permission types
// note that ESTUSERS and ESTCHANNELS must be 2^x-1
#define ESTUSERS	65535	// optimize this for your network
#define ESTCHANNELS	16383	// optimize this for your network
#define MAXCHANLEN	255
#define MAXNICKLEN	31
#define MAXTOPICLEN	255
#define MAXHOSTLEN	255
#define MAXREALLEN	255
#define CAP_QS  	0x00000001
#define CAP_EX  	0x00000002
#define CAP_CHW 	0x00000004
#define CAP_DE  	0x00000008
#define	CAP_MY		0x0000000F
#define TS_CURRENT	3	// TS version we are using
#define TS_MIN		3	// minTS version we are supporting
#define MODESPERLINE	99	// this is ircd-hybrid standard
#define	BIGMODESPERLINE	13	// this is ircd-hybrid standard
#define FUCKOVER_TIME	20	// max. fuckover time in seconds

#define context set_context(__FILE__, __LINE__)

struct TChannel;
struct TChanList;

typedef struct _TCapab
 {
  char *c;
  int capab;
 } TCapab;

typedef struct _TChanL
 {
  struct _TChannel *channel;
  struct _TChanL *next, *prev;
 } TChanL;

typedef struct _TServer
 {
  char name[MAXHOSTLEN+1], desc[MAXREALLEN+1];
  time_t ts;
  int hops;
  int users, maxusers;
  double avgusers;
  time_t maxusersTS, avgusersTS;
  int internal; // 0=ext., 1=services, 2=juped
  int temp;     // temp for deleting servers
  struct _TServer *next, *uplink;
 } TServer;

typedef struct _TUser
 {
  char user[MAXHOSTLEN+1], host[MAXHOSTLEN+1], name[MAXREALLEN+1];
  char nick[MAXNICKLEN+1];
  time_t ts;
  int ircop;                    // IRCOP ?
  int invisible;                // invisible (+i) ?
  int hops;
  int internal;			// 0=ext., 1=services, 2=juped
  int clnumber;			// clone no.
  struct _TServer *serv;	// server on which user is
  struct _TUser *next, *prev;	// user next in hashtable
  struct _TChanL *chans;	// channel where user is
 } TUser;

typedef struct _THashUser
 {
  struct _TUser* hashtable[ESTUSERS];
  int users, maxusers, lusers, maxlusers;
  double avgusers, avglusers;
  time_t maxusersTS, maxlusersTS, avgusersTS, avglusersTS;
  int invisible, ircop;
 } THashUser;

typedef struct _TNickList
 {
  int op, vop;
  struct _TUser *nick;
  struct _TNickList *next, *prev;
 } TNickList;

typedef struct _TChanList
 {
  char channel[MAXCHANLEN+1];
  struct _TChanList *next, *prev;
 } TChanList;

typedef struct _TBanList
 {
  char ban[MAXCHANLEN+1];
  struct _TBanList *next, *prev;
 } TBanList;

typedef struct _TUH
 {
  char userhost[2*MAXHOSTLEN+2];
  int count;
  struct _TNickList *nicklist;
  struct _TUH *next, *prev;
 } TUH;

typedef struct _THashUserHost
 {
  struct _TUH* hashtable[ESTUSERS];
  int userhosts, maxuserhosts;
  double avguserhosts;
  time_t maxuserhostsTS, avguserhostsTS;
 } THashUserHost;

typedef struct _TH
 {
  char host[MAXHOSTLEN+1];
  int count;
  struct _TNickList *nicklist;
  struct _TH *next, *prev;
 } TH;

typedef struct _THashHost
 {
  struct _TH* hashtable[ESTUSERS];
  int hosts, maxhosts;
  double avghosts;
  time_t maxhostsTS, avghostsTS;
 } THashHost;

typedef struct _TChannel
 {
  char name[MAXCHANLEN+1];
  char topic[MAXTOPICLEN+1];
  int users;
  int inviteo, userlimit, moderated, nomsg, private, secure, topicopo;
  char key[MAXREALLEN+1];
  time_t ts;
  struct _TNickList *nicklist;
  struct _TBanList *banlist;
  struct _TBanList *exclist;
  struct _TBanList *regexdeny;
  struct _TChannel *next, *prev;	// channel next in hashtable
 } TChannel;

typedef struct _THashChannel
 {
  struct _TChannel* hashtable[ESTCHANNELS];
  int channels, maxchannels;
  double avgchannels;
  time_t maxchannelsTS, avgchannelsTS;
 } THashChannel;

typedef int TPerm[PERMNO];

typedef struct _TOper
 {
  char nick[MAXNICKLEN+1], pass[MAXREALLEN+1], host[2*MAXHOSTLEN+2];
  TPerm perms;
  struct _TUser *lognick;
  struct _TChanList *chanlist;
  struct _TOper *next;
 } TOper;

typedef struct _TConn
 {
  char host[MAXHOSTLEN+1], name[MAXHOSTLEN+1], pass[MAXREALLEN+1];
  int port;
  int wildcarding;
  struct _TConn *next;
 } TConn;

typedef struct _TConfig
 {
  char name[MAXHOSTLEN+1], desc[MAXREALLEN+1], admin[3][MAXREALLEN+1];
  char nick[MAXNICKLEN+1];
  int fork;
  int uselog;
  struct _TConn *conns;
  struct _TOper *opers;
 } TConfig;

typedef struct _TCommand
 {
  char *cmd;
  int (*handler)(void);
  int perm;
  char *helpstr;
  char *help2str;
 } TCommand;

typedef struct _TCmdInput
 {
  char *cmd;
  void (*handler)(void);
 } TCmdInput;

typedef struct _TInfo
 {
  int servers,maxservers;
  double avgservers;
  time_t maxserversTS, avgserversTS;
  int kills,joins,nickchngs,chanjupes,servjupes,nickjupes,clones;
  time_t servicestart,linktime;
  TConn *linkserver;
  int linkcapab;
 } TInfo;

typedef struct _TFOLst
 {
#ifndef NO_PTHREAD
  pthread_t fuckover_thread;
#else
  int fuckover_thread;
#endif // NO_PTHREAD
  int stop;
  struct _TUser *user;
  struct _TFOLst *next, *prev;
 } TFOLst;
 
TConfig config;
THashUser *users;
THashUserHost *userhosts;
THashHost *hosts;
TServer *servers;
THashChannel *channels;
TInfo info;
TFOLst *folist;
int disconnect;

int upFD, args, rec_HUP, rec_TERM, segv;
char buffer[1024];
char *who, *whom, *arg[512];
TFOLst *foltmp;
extern TCommand commands[];

// misc.c functions
double calcavg(double, double, time_t);
char *makestring(char *, const char *, ...);
char *getdate(time_t);
int max(int, int);
void lineargs(int);
void logit(const char *, ...);
void set_context(char *, int);
void fatal(int);
int match(const unsigned char *, const unsigned char *);

// network.c functions
void sockprintf(int, char *, ...);
int connectto(TConn *);
void breaklink(char *);
void sendserver(int, TServer *);
void senduser(int, TUser *);

// server.c functions
void usernotice(char *, char *, ...);
void usermsg(char *, char *, ...);
void wallop(char *, ...);
void op(char *, char *, char *);
void deop(char *, char *, char *);
void invite(char *, char *, char *);
void unban(char *, char *, char *);
void mode(char *, char *, char *, int);
void kick(char *, char *, char *, char *);
void killl(char *, char *, char *);
void collide(char *, char *);
void s_connect(char *, char *, int i, char *);
void squit(char *, char *, char *);
void privmsg(char *, char *, char *);
void notice(char *, char *, char *);
void install_jupe(char *, char *, char *);
void remove_jupe(char *, char *);
void quit(char *, char *, char *);

// users.c functions
unsigned int hashuser(const char *);
unsigned int hashchannel(const char *);
TUser *finduser(char *);
TOper *findcnoper(char *);
TOper *findlogoper(char *, char *, char *);
TChannel *findchannel(char *);
TNickList *finduseronchan(TChannel *, char *);
int matchchannel(char *, TOper *);
void delchannel(TChannel *);
void delchanuser(TChannel *, TUser *);
void delserver(TServer *);
void adduser(char *nick, char *user, char *host, char *server, time_t ts, int hops, int ircop, int invisible, int internal, char *name, ...);
TChannel *addchannel(char *name, int);
int setmode(TChannel *, char *, int);
void addchanuser(TChannel *, TUser *, int, int);
void addserver(char *, char *, int, int, char *);
void delmyclones(TUser *);
void deluser(TUser *);
TServer *findserver(char *);

// cmdhandle.c functions
/*void h_help(void);
void h_stat(void);
void h_auth(void);
void h_op(void);
void h_mode(void);
void h_kick(void);
void h_kill(void);
void h_jupe(void);
void h_mkclon(void);
*/
// confile.c functions
void load_config(void);

// inputhandle.c functions
void hh_parse(char *);

// capab.c functions
char *mycapab(char *);
int checkcapab(char *);

// password.c functions
char *pcrypt(char *);
int passcheck(char *, char *);

#endif /* _IRCDSERV_H */
