/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
#include "ircdserv.h"

FILE *LOGfile;
int context_line=0;
char context_file[256];

static char* months[] =
 {
  "January", "February", "March", "April",
  "May", "June", "July", "August",
  "September", "October", "November", "December"
 };
      

char *makestring(char *buffer, const char *format, ...)
 {
  va_list va;
  va_start(va, format);
  vsprintf(buffer, format, va);
  va_end(va);
  context;
  return(buffer);
 }

char *getdate(time_t clock)
 {
  static char buf[80];
  struct tm *lt, *gm;
  struct tm gmbuf;

  context;
  if (!clock)
    time(&clock);
  gm = gmtime(&clock);
  memcpy((void *)&gmbuf, (void *)gm, sizeof(gmbuf));
  gm = &gmbuf;
  lt = localtime(&clock);
  context;
  return makestring(buf, "%02i:%02i:%02i %s %i", lt->tm_hour, lt->tm_min, lt->tm_sec, months[lt->tm_mon], lt->tm_mday);
 }

double calcavg(double oldavg, double current, time_t oldtime)
 {
  time_t timenow;
  double i;
  
  context;
  timenow=time(NULL);
  if(timenow-info.servicestart)
    i=(oldavg*(oldtime-info.servicestart)+current*(timenow-oldtime))/(timenow-info.servicestart);
   else i=current;
  context;
  return i;
 }

int max(int a, int b)
 {
  context;
  if(a>b) return a; else return b;
 }

void lineargs(int i)
 {
  int j;
  
  context;
  j=i;
  while(j<args)
   {
    context;
    arg[j][strlen(arg[j])]=' ';
    j++;
   }
 }

FILE *logfd;
 
void logit(const char *format, ...)
 {
  va_list va;
  char buffer[1024], temp[1024];
  time_t now=time(NULL);
  va_start(va, format);
  vsnprintf(buffer, 1024, format, va);
  va_end(va);
  if(config.uselog)
   {
    strncpy(temp, ctime(&now), 1024);
    if(strchr(temp, '\n'))
    *strchr(temp, '\n') = '\0';
    fprintf(logfd,"[%s] %s", temp, buffer);
    fflush(logfd);
   }
  if(!config.fork)
   {
    printf(buffer);
    fflush(stdout);
   }
 }

void fatal(int i)
 {
  logit("At least one fatal error found, terminating...\n");
  exit(i);
 }

void set_context(char *file, int line)
 {
  context_line = line;
  strncpy(context_file, file, 256);
 }

/* taken by pod from BitchX source.. */
int match (const unsigned char *pattern, const unsigned char *string)
 {
  int		count=1;
  int 		asterisk=0;
  int		percent=0;
  const char	*last_asterisk_point=NULL;
  const char	*last_percent_point=NULL;
  int		last_asterisk_count=0;
  int		last_percent_count=0;
  const char	*after_wildcard=NULL;
  int		sanity=0;
  const char	*old_pattern=pattern, *old_string=string;
 
  context;
  for (;;)
   {
    if (sanity++>100000)
     {
      logit("Infinite loop in match! pattern = [%s] string = [%s]", old_pattern, old_string);
      return 0;
     }
    if (asterisk)
     {
      if (!*string)
      return 0;
      if (*pattern=='\\')
       {
        pattern++;
        if (tolower(*string)!=tolower(*pattern)) continue;
       } else
      if (*pattern=='?'||(tolower(*string)==tolower(*pattern)))
       {
        asterisk=0;
        last_asterisk_point=string;
        last_asterisk_count=count;
       } else string++;
      continue;
     }
    if (percent)
     {
      if (!*string)
       {
        if (*pattern) return 0; else return count;
       }
      if (*pattern=='\\')
       {
        pattern++;
        if (tolower(*string)!=tolower(*pattern)) continue;
       }
      if (*string==' ')
       {
        percent=0;
        last_percent_point=NULL;
       } else
      if (tolower(*string)!=tolower(*pattern)) string++; else
       {
        percent=0;
        last_percent_point=string;
        last_percent_count=count;
       }
      continue;
     }
    switch (*pattern)
     {
      case ('*') : case ('%') :
       {
        asterisk=0,percent=0;
        do
         {
          if (*pattern=='*')
          asterisk=1;
          pattern++;
         }
        while (*pattern=='*' || *pattern=='%');
        after_wildcard=pattern;
        if (asterisk)
         {
          last_asterisk_point=string;
          last_asterisk_count=count;
         } else
         {
          percent=1;
          last_percent_point=string;
          last_percent_count=count;
         }
        if (asterisk && !*pattern) return count;
        break;
       }
      case ('?') :
       {
        pattern++;
        if (!*string) return 0;
        string++;
        break;
       }
      case ('\\') :
       {
        pattern++;
        if (!*pattern) return 0;
        if (tolower(*pattern)!=tolower(*string)) return 0;
        count++, string++, pattern++;
        break;
       }
      default:
       {
        if (!*pattern && !*string) return count;
        if (tolower(*pattern) == tolower(*string))
         {
          count++, pattern++, string++;
         } else
        if (last_asterisk_point)
         {
          asterisk=1;
          string=last_asterisk_point+1;
          pattern=after_wildcard;
          count=last_asterisk_count;
         } else
        if (last_percent_point)
         {
          percent=1;
          string=last_percent_point+1;
          pattern=after_wildcard;
          count=last_percent_count;
         } else return 0;
        break;
       }
     }
   }
  context;
  return 0;
 }
