/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
#include "ircdserv.h"

void usernotice(char *nick, char *msg, ...)
 {
  char buffer[1024];
  va_list va;
  context;
  va_start(va, msg);
  vsprintf(buffer, msg, va);
  va_end(va);
  sockprintf(upFD, ":%s NOTICE %s :%s\n", config.nick, nick, buffer);
  context;
 }

void usermsg(char *nick, char *msg, ...)
 {
  char buffer[1024];
  va_list va;
  context;
  va_start(va, msg);
  vsprintf(buffer, msg, va);
  va_end(va);
  sockprintf(upFD, ":%s PRIVMSG %s :%s\n", config.nick, nick, buffer);
 }

void wallop(char *msg, ...)
 {
  char buffer[1024];
  va_list va;
  context;
  va_start(va, msg);
  vsprintf(buffer, msg, va);
  va_end(va);
  sockprintf(upFD, ":%s WALLOPS :%s\n", config.nick, buffer);
 }

void op(char *who, char *chan, char *whom)
 {
  TChannel *channel;
  TNickList *nick;

  context;
  if(!(channel=findchannel(chan)))
    usernotice(who, "No such channel - %s.", chan); else
  if(!(nick=finduseronchan(channel,whom)))
    usernotice(who, "%s is not on channel %s.", whom, chan); else
  if(nick->op)
    usernotice(who, "%s is already op on %s.", whom, chan); else
   {
    nick->op=1;
    sockprintf(upFD, ":%s MODE %s +o %s\n", config.nick, chan, whom);
#ifdef LOG_OP
    logit("OP: %s requested for +o %s on %s.\n", who, whom, chan);
#endif
   }
 }

void deop(char *who, char *chan, char *whom)
 {
  TChannel *channel;
  TNickList *nick;

  context;
  if(!(channel=findchannel(chan)))
    usernotice(who, "No such channel - %s.", chan); else
  if(!(nick=finduseronchan(channel,whom)))
    usernotice(who, "%s is not on channel %s.", whom, chan); else
  if(!nick->op)
    usernotice(who, "%s is not op on %s.", whom, chan); else
   {
    nick->op=0;
    sockprintf(upFD, ":%s MODE %s -o %s\n", config.nick, chan, whom);
#ifdef LOG_OP
    logit("DEOP: %s requested for -o %s on %s.\n", who, whom, chan);
#endif
   }
  context;
 }

void invite(char *who, char *chan, char *whom)
 {
  TChannel *channel;

  context;
  if(!(channel=findchannel(chan)))
    usernotice(who, "No such channel - %s.", chan); else
  if(!finduser(whom))
    usernotice(who, "No such nick - %s.", whom); else
  if(finduseronchan(channel,whom))
    usernotice(who, "%s is already on channel %s.", whom, chan); else
   {
    sockprintf(upFD, ":%s SJOIN %i %s + :@%s\n", config.name, channel->ts, chan, config.nick);
    sockprintf(upFD, ":%s INVITE %s :%s\n", config.nick, whom, chan);
    sockprintf(upFD, ":%s PART %s\n", config.nick, chan);
#ifdef LOG_OP
    logit("INVITE: %s requested for INVITE %s on %s.\n", who, whom, chan);
#endif
   }
 }

void unban(char *who, char *chan, char *whom)
 {
  TChannel *channel;
  TUser *user;
  TBanList *ban;
  int removed;
  char userhost[MAXNICKLEN+2*MAXHOSTLEN+3];

  context;
  if(!(channel=findchannel(chan)))
    usernotice(who, "No such channel - %s.", chan); else
  if(!(user=finduser(whom)))
    usernotice(who, "No such nick - %s.", whom); else
   {
    sprintf(userhost, "%s!%s@%s", user->nick, user->user, user->host);
    for(removed=0, ban=channel->banlist; ban; ban=ban->next)
     if(match(ban->ban, userhost))
     {
// zislo by sa tie bany aj vyhodit z databazy...
      removed++;
      sockprintf(upFD, ":%s MODE %s -b %s\n", config.nick, chan, ban->ban);
     }
    for(ban=channel->regexdeny; ban; ban=ban->next)
     if(match(ban->ban, user->name))
     {
      removed++;
      sockprintf(upFD, ":%s MODE %s -d %s\n", config.nick, chan, ban->ban);
     }
    if(removed) usernotice(who, "Total %i bans removed.", removed); else
                usernotice(who, "%s is not banned on %s.", whom, chan);
#ifdef LOG_OP
    logit("UNBAN: %s requested for -b %s on %s.\n", who, whom, chan);
#endif
   }
 }

void mode(char *who, char *chan, char *mode, int j)
 {
  TChannel *channel;
  TNickList *nick;
  int i=0, sign=1;
  char mode1[256], mode2[256];
  context;
  if(chan[0]!='#')
    usernotice(who, "Can't change usermode for %s.", chan); else
  if(!(channel=findchannel(chan)))
    usernotice(who, "No such channel - %s.", chan); else
   {
    mode1[0]=0; mode2[0]=0;
    while(mode[i])
     {
      switch(mode[i])
       {
        case 'b': case 'k': case 'l': case 'e': case 'd':
         {
          if(j>args)
           {
            usernotice(who, "Not enought parameters. Truncating...");
            i=strlen(mode)-1;
           } else
           {
            sprintf(mode1, "%s%c", mode1, mode[i]);
            sprintf(mode2, "%s %s", mode2, arg[j++]);
            context;
           }
          break;
         }
        case 'i': case 'm': case 'n': case 'p': case 's': case 't': case '+': case '-':
         {
          if(mode[i]=='+') sign=1; else
          if(mode[i]=='-') sign=0;
          context;
          mode1[strlen(mode1)+1]=0;
          mode1[strlen(mode1)]=mode[i];
          break;
         }
        case 'o':
         {
          if(j>args)
           {
            usernotice(who, "Not enought parameters. Truncating...");
            i=strlen(mode)-1;
           } else
           {
            context;
            if(!(nick=finduseronchan(channel, arg[j])))
              usernotice(who, "%s is not on channel %s. Skipping.", arg[j], chan); else
            if(nick->op && sign)
              usernotice(who, "%s is already op on %s. Skipping.", arg[j], chan); else
            if(!nick->op && !sign)
              usernotice(who, "%s is not op on %s. Skipping.", arg[j], chan); else
             {
              if(sign) nick->op=1; else nick->op=0;
              sprintf(mode1, "%s%c", mode1, mode[i]);
              sprintf(mode2, "%s %s", mode2, arg[j]);
             }
           }
          j++;
          break;
         }
        case 'v':
         {
          if(j>args)
           {
            context;
            usernotice(who, "Not enought parameters. Truncating...");
            i=strlen(mode)-1;
           } else
           {
            context;
            if(!(nick=finduseronchan(channel,arg[j])))
              usernotice(who, "%s is not on channel %s. Skipping.", arg[j], chan); else
            if((nick->op || nick->vop) && sign)
              usernotice(who, "%s is already (v)op on %s. Skipping.", arg[j], chan); else
            if(!nick->vop && !sign)
              usernotice(who, "%s is not vop on %s. Skipping.", arg[j], chan); else
             {
              if(sign) nick->vop=1; else nick->vop=0;
              sprintf(mode1, "%s%c", mode1, mode[i]);
              sprintf(mode2, "%s %s", mode2, arg[j]);
             }
           }
          context;
          j++;
          break;
         }
       }
      context;
      i++;
     }
    context;
    sockprintf(upFD, ":%s MODE %s %s%s\n", config.nick, chan, mode1, mode2);
#ifdef LOG_MODE
    logit("%s requested for %s %s on %s.\n", who, mode1, mode2, chan);
#endif
   }
 }

void kick(char *who, char *chan, char *nick, char *reason)
 {
  TChannel *channel;
  TNickList *nnick;

  context;
  if(!(channel=findchannel(chan)))
    usernotice(who, "No such channel - %s.", chan); else
  if(!(nnick=finduseronchan(channel, nick)))
    usernotice(who, "%s is not on channel %s.", whom, chan); else
   {
    delchanuser(channel, finduser(nick));
    sockprintf(upFD, ":%s KICK %s %s :%s\n", config.nick, chan, nick, reason);
#ifdef LOG_KICK
    logit("%s requested kick for %s on %s (%s).\n", who, nick, chan, reason);
#endif
   }
 }

void killl(char *who, char *nick, char *reason)
 {
  TUser *user;
  
  context;
  if(!(user=finduser(nick)))
    usernotice(who, "No such nick - %s.", nick); else
  if(user->ircop && who)
    usernotice(who, "Can't kill an IRC operator."); else
   {
    info.kills++;
    deluser(user);
    sockprintf(upFD, ":%s KILL %s :%s!%s\n", config.nick, nick, config.name, reason);
#ifdef LOG_KILL
    logit("%s requested kill for %s (%s).\n", who, nick, reason);
#endif
   }
 }

void quit(char *who, char *nick, char *reason)
 {
  TUser *user;
  
  context;
  if(!(user=finduser(nick)))
    usernotice(who, "No such nick - %s.", nick); else
   {
    info.kills++;
    deluser(user);
    sockprintf(upFD, ":%s QUIT %s :%s\n", nick, reason);
   }
 }
 
void collide(char *who, char *nick)
 {
  TUser *user;
  
  context;
  if(!(user=finduser(nick)))
    usernotice(who, "No such nick - %s.", nick); else
  if(user->ircop && who)
    usernotice(who, "Can't collide an IRC operator."); else
   {
    info.kills++;
    sockprintf(upFD, "NICK %s 1 %d + services %s %s :Nick Collision\n", nick,  user->ts, config.name, config.name);
    deluser(user);
   }
 }

void s_connect(char *who, char *what, int port, char *uplink)
 {
  TServer *server;
  
  context;
  if(!(server=findserver(uplink)))
   usernotice(who, "No such server - %s.", server); else
   sockprintf(upFD, ":%s CONNECT %s %i :%s\n", config.nick, what, port, uplink);
 }

void squit(char *who, char *serv, char *reason)
 {
  TServer *server;
 
  context;
  if(!(server=findserver(serv)))
   usernotice(who, "No such server - %s.", server); else
   {
    sockprintf(upFD, ":%s SQUIT %s :%s\n", config.nick, serv, reason);
#ifdef LOG_SQUIT
    logit("%s requested squit for %s (%s).\n", who, serv, reason);
#endif
    delserver(server);
   }
 }

void privmsg(char *who, char *whom, char *msg)
 {
  if(!finduser(whom))
   usernotice(who, "No such nick - %s.", whom); else
   sockprintf(upFD, ":%s PRIVMSG %s :%s", config.nick, whom, msg);
 }

void notice(char *who, char *whom, char *msg)
 {
  if(!finduser(whom))
   usernotice(who, "No such nick - %s.", whom); else
   sockprintf(upFD, ":%s NOTICE %s :%s", config.nick, whom, msg);
 }

