/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
/*
 * modul socketovych operacii
 *
 * ! pri strate spojenia tato kniznica generuje signal SIGUSR1 pre !
 * ! proces v ktorom bezi!                                         !
 *
 */

#include "ss_sock.h"
#include "dns.h"

#define DEFSOCKTIMEOUT_S 10
#define DEFSOCKTIMEOUT_MS 10

int sock_timeout_s = DEFSOCKTIMEOUT_S,
    sock_timeout_ms = DEFSOCKTIMEOUT_MS;

int set_sock_option( int fd, int option, int arg )
{
    int arg_x;
    
    arg_x = arg;
    
    return setsockopt( fd, SOL_SOCKET, option, &arg_x, sizeof( arg_x ));
}

/* otvori socket */
int open_sock( void )
{
    int sock;

    /* Vytvor socket v Internet domene, prudovy a TCP/IP */
    sock = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
    set_sock_option( sock, SO_KEEPALIVE, 1 );
    return sock;
}

/* bindne socket na adresu IP alebo domenovy nazov a vrati port socketu
 * alebo -1 pri chybe
 *
 * ked mode je 0, je to klient
 * ked 1 tak server (listen)
 * ked port je 0, tak vrati prvy volny
 *
 * __addx moze byt ciselna adresa alebo domenovy nazov
 *
 */
int bind_sock( int fd, char * __addx, int port, int mode )
{
    int saddr_size, newport;
    struct sockaddr_in saddr, saddr1;
    struct hostent *he;

    /* Prirad IP adresu k socketu */
    saddr_size = sizeof( saddr );

    saddr.sin_family = AF_INET;
    saddr.sin_port = htons( port );
    saddr.sin_addr.s_addr = INADDR_ANY;

    /* ked je addx NULL tak sa pouzije default INADDR_ANY */
    if( __addx )
    {
	/* zisti ci ide o IP adresu */
        if( inet_aton( __addx, &saddr.sin_addr ) == 0 ) 
        {
    	    /* zisti ci ide o domenovy nazov a ak ano tak resolvuj */
    	    he = gethostbyname( __addx );
	    if( !he )
		return -1;
	    /* pouzi tuto adresu (prvu v zozname moznych aliasov pre dany host) */
	    bcopy( he -> h_addr, &saddr.sin_addr, he -> h_length );
	}
    }

    if( mode ) /* strana servra (pocuvanie) */
    {
	/* binduj nazov k socketu */
	if( bind( fd, (struct sockaddr*) &saddr, saddr_size ) == -1 )
	    return -1;

        /* zisti skutocny port a adresu socketu */
        if( getsockname( fd, (struct sockaddr*) &saddr1, &saddr_size ) < 0 )
    	    return -1;

        newport = ntohs( saddr1.sin_port );

        /* uved socket do nacuvacieho ("server") rezimu */
	if( listen( fd, 2 ) < 0 )
	    return -1;
    } else { /* strana klienta */
	newport = connect( fd, (struct sockaddr*) &saddr, saddr_size );
    }

    return newport;
}

int connect_sock_ip( int fd, struct in_addr* addr, int port )
{
    struct sockaddr_in sa;
    
    sa.sin_family = AF_INET;
    sa.sin_port = htons( port );
    sa.sin_addr = *addr;

    return connect( fd, (struct sockaddr*) &sa, sizeof( sa ) );
}

int connect_sock_hostname( int fd, char *hostname, int port )
{
    struct in_addr ia;

    if( mygethostbyname( hostname, &ia ) == -1 )
	return -1;
    return connect_sock_ip( fd, &ia, port );
}

int close_sock( int fd )
{
    return close( fd );
}

/* vrati 1 ked nastal na deskriptore timeout viac ako zadany alebo -1 
 * alebo 0 ked je to okej 
 */
int timeout_write( int fd, int secs, int msecs )
{
    int retval;
    fd_set descr;
    struct timeval tv;

    FD_ZERO( &descr );
    FD_SET( fd, &descr );
    tv.tv_sec = secs;
    tv.tv_usec = msecs;

    retval = select( fd + 1, NULL, &descr, NULL, &tv );

    if( retval < 1 )
	return retval;

    if( FD_ISSET( fd, &descr ) )
	return 0;

    return 1;
}

/* to iste ale pre citanie */
int timeout_read( int fd, int secs, int msecs )
{
    int retval;
    fd_set descr;
    struct timeval tv;

    FD_ZERO( &descr );
    FD_SET( fd, &descr );
    tv.tv_sec = secs;
    tv.tv_usec = msecs;

    retval = select( fd + 1, &descr, NULL, NULL, &tv );

    if( retval < 1 )
	return -1;

    if( FD_ISSET( fd, &descr ) )
	return 0;

    return 1;
}

/* zapise data a vrati pocet zapisanych alebo -1 v pripade chyby
 * alebo -2 ked nastane timeout pri zapise... vrati 0 ked peer died
 */

int secure_write( int fd, void * buff, size_t size )
{
    int count;

    if( timeout_write( fd, sock_timeout_s, sock_timeout_ms ) )
	return -2;

    if( (count = send( fd, buff, size, 0)) == -1 ) 
	return -1;

/*
    if( !count )
	raise( SIGUSR1 );
*/

    return count;
}

int secure_send_string( int fd, char * str )
{
    return secure_write( fd, str, strlen( str ) );
}

/* vrati pocet precitanych bajtov
 * alebo -1 v pripade chyby alebo 0 v pripade timeoutu 
 * no a -2 ked peer died
 */
int secure_read( int fd, void * buff, size_t size )
{
    int count;

    if( timeout_read( fd, sock_timeout_s, sock_timeout_ms ) )
	return 0;

    if( (count = recv( fd, buff, size, 0 )) == -1 )
	return -1;

/*
    if( !count )
	raise( SIGUSR1 );
*/
    if(!count) return -2;
    return count;
}


int init_sock( void )
{
    return 0;
}

int done_sock( void )
{
    return 0;
}

