/***********************************************************************
 *   IRCDSERV - Internet Relay Chat Services                            *
 *   Copyright (C) 1999-2000 Jan Oravec(WSX) and Juraj Lutter(otis)     *
 *                                                                      *
 *   This program is free software; you can redistribute it and/or      *
 *   modify it under the terms of the GNU General Public License        *
 *   as published by the Free Software Foundation; either version 1,    *
 *   or (at your option) any later version.                             *
 *                                                                      *
 *   This program is distributed in the hope that it will be useful,    *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *   GNU General Public License for more details.                       *
 *                                                                      *
 *   You should have received a copy of the GNU General Public License  *
 *   along with this program; if not, write to the Free Software        *
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.          *
  ***********************************************************************/
#include "ircdserv.h"

const unsigned char ToLowerTab[] = { 
  0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
  0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
  0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
  0x1e, 0x1f,
  ' ', '!', '"', '#', '$', '%', '&', 0x27, '(', ')',
  '*', '+', ',', '-', '.', '/',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  ':', ';', '<', '=', '>', '?',
  '@', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
  'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
  '_',
  '`', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
  'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
  0x7f,
  0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
  0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
  0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99,
  0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
  0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
  0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
  0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9,
  0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
  0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9,
  0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
  0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
  0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
  0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
  0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
  0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
  0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

#define ToLower(c) (ToLowerTab[(unsigned char)(c)])

unsigned int hashuser(const char *c)
 {
  unsigned int i=0;
  context;
  while(*c) i=(i<<4)-(i+(unsigned char)ToLower(*c++));
  context;
  return(i&ESTUSERS);
 }

unsigned int hashchannel(const char *c)
 {
  unsigned int i=0;
  context;
  while(*c) i=(i<<4)-(i+(unsigned char)ToLower(*c++));
  return(i&ESTCHANNELS);
 }

TUser *finduser(char *nick)
 {
  TUser *tmpuser;
  
  context;
  if(!nick) return NULL;
  tmpuser=users->hashtable[hashuser(nick)];
  while(tmpuser && strcasecmp(tmpuser->nick, nick)) tmpuser=tmpuser->next;
  context;
  return tmpuser;
 }

TChannel *findchannel(char *channel)
 {
  TChannel *tmpchan;
  
  context;
  if(!channel) return NULL;
  tmpchan=channels->hashtable[hashchannel(channel)];
  while(tmpchan && strcasecmp(tmpchan->name, channel)) tmpchan=tmpchan->next;
  return tmpchan;
 }

TNickList *finduseronchan(TChannel *chan, char *nick)
 {
  TNickList *tmpnick;

  context;
  if(!chan || !nick) return NULL;
  for(tmpnick=chan->nicklist;tmpnick;tmpnick=tmpnick->next)
    if(!strcasecmp(tmpnick->nick->nick,nick)) return tmpnick;
  return NULL;
 }

TOper *findlogoper(char *nick, char *user, char *host)
 {
  TOper *tmpoper;
  char uh[1024];
  
  context;
  if(!nick || !user || !host) return NULL;
  sprintf(uh, "%s@%s", user, host);
  for(tmpoper=config.opers;tmpoper;tmpoper=tmpoper->next)
    if((!strcasecmp(tmpoper->nick, nick)) && (match(tmpoper->host, uh))) return tmpoper;
  return NULL;
 }

TServer *findserver(char *server)
 {
  TServer *tmpserv;

  context;
  if(!server) return NULL;
  for(tmpserv=servers;tmpserv;tmpserv=tmpserv->next)
    if(!strcasecmp(tmpserv->name, server)) return tmpserv;
  return NULL;
 }
/*
TServer *findmatchserver(char *server)
 {
  TServer *tmpserv;

  context;
  if(!server) return NULL;
  for(tmpserv=servers;tmpserv;tmpserv=tmpserv->next)
    if(match(tmpserv->name, server) || match(server, tmpserv->name)) return tmpserv;
  return NULL;
 }
*/
TOper *findcnoper(char *nick)
 {
  TOper *tmpoper;
  
  context;
  for(tmpoper=config.opers;tmpoper;tmpoper=tmpoper->next)
    if((tmpoper->lognick) && !strcasecmp(tmpoper->lognick->nick,nick)) return tmpoper;
  return NULL;
 }

int matchchannel(char *channel, TOper *oper)
 {
  TChanList *tmpchan;
  
  context;
  if(!oper && !channel) return 0;
  for(tmpchan=oper->chanlist;tmpchan;tmpchan=tmpchan->next)
    if(match(tmpchan->channel,channel)) return 1;
  return 0;
 }

void deluserhost(char *c, TUser *user)
 {
  TUH *tuh;
  TNickList *nlist;
  
  context;
  for(tuh=userhosts->hashtable[hashuser(c)]; tuh; tuh=tuh->next)
   if(!strcasecmp(c, tuh->userhost))
    {
     tuh->count--;
     nlist=tuh->nicklist;
     while(nlist && (nlist->nick!=user)) nlist=nlist->next;
     if(!nlist)
      {
       logit("DELUSERHOST: Nicklist entry not found.");
       fatal(2);
      }
     if(nlist->next) nlist->next->prev=nlist->prev;
     if(nlist->prev) nlist->prev->next=nlist->next; else
      tuh->nicklist=nlist->next;
     free(nlist);
     if(!tuh->count)
      {
       if(tuh->next) tuh->next->prev=tuh->prev;
       if(tuh->prev) tuh->prev->next=tuh->next; else
        userhosts->hashtable[hashuser(c)]=tuh->next;
       userhosts->avguserhosts=calcavg(userhosts->avguserhosts, userhosts->userhosts, userhosts->avguserhostsTS);
       userhosts->avguserhostsTS=time(NULL);
       userhosts->userhosts--;
       free(tuh);
      }
     context;
     return;
    }
  logit("DELUSERHOST: user@host %s not found.\n", c);
  fatal(2);
 }

void delhost(char *c, TUser *user)
 {
  TH *th;
  TNickList *nlist;
  
  context;
  for(th=hosts->hashtable[hashuser(c)]; th; th=th->next)
   if(!strcasecmp(c, th->host))
    {
     th->count--;
     nlist=th->nicklist;
     while(nlist && (nlist->nick!=user)) nlist=nlist->next;
     if(!nlist)
      {
       logit("DELHOST: Nicklist entry not found.");
       fatal(2);
      }
     if(nlist->next) nlist->next->prev=nlist->prev;
     if(nlist->prev) nlist->prev->next=nlist->next; else
      th->nicklist=nlist->next;
     free(nlist);
     if(!th->count)
      {
       if(th->next) th->next->prev=th->prev;
       if(th->prev) th->prev->next=th->next; else
        hosts->hashtable[hashuser(c)]=th->next;
       hosts->avghosts=calcavg(hosts->avghosts, hosts->hosts, hosts->avghostsTS);
       hosts->avghostsTS=time(NULL);
       hosts->hosts--;
       free(th);
      }
     context;
     return;
    }
  logit("DELHOST: host %s not found.\n", c);
  fatal(2);
 }

int adduserhost(char *c, TUser *user)
 {
  TUH *tuh;
  TNickList *nlist;
  
  context;
  tuh=userhosts->hashtable[hashuser(c)];
  for(; tuh; tuh=tuh->next)
   if(!strcasecmp(c, tuh->userhost))
    {
     tuh->count++;
     nlist=malloc(sizeof(TNickList));
     nlist->prev=NULL;
     nlist->next=tuh->nicklist;
     nlist->next->prev=nlist;
     nlist->nick=user;
     tuh->nicklist=nlist;
     return tuh->count;
    }
  userhosts->avguserhosts=calcavg(userhosts->avguserhosts, userhosts->userhosts, userhosts->avguserhostsTS);
  userhosts->avguserhostsTS=time(NULL);
  userhosts->userhosts++;
  if(userhosts->userhosts>userhosts->maxuserhosts)
   {
    userhosts->maxuserhosts=userhosts->userhosts;
    userhosts->maxuserhostsTS=time(NULL);
   }
  tuh=malloc(sizeof(TUH));
  strcpy(tuh->userhost, c);
  tuh->next=userhosts->hashtable[hashuser(c)];
  tuh->prev=NULL;
  if(tuh->next) tuh->next->prev=tuh;
  userhosts->hashtable[hashuser(c)]=tuh;
  tuh->count=1;
  nlist=malloc(sizeof(TNickList));
  nlist->prev=NULL;
  nlist->next=NULL;
  nlist->nick=user;
  tuh->nicklist=nlist;
  context;
  return tuh->count;
 }

int addhost(char *c, TUser *user)
 {
  TH *th;
  TNickList *nlist;
  
  th=hosts->hashtable[hashuser(c)];
  for(; th; th=th->next)
   if(!strcasecmp(c, th->host))
    {
     th->count++;
     nlist=malloc(sizeof(TNickList));
     nlist->prev=NULL;
     nlist->next=th->nicklist;
     nlist->next->prev=nlist;
     nlist->nick=user;
     th->nicklist=nlist;
     return th->count;
    }
  hosts->avghosts=calcavg(hosts->avghosts, hosts->hosts, hosts->avghostsTS);
  hosts->avghostsTS=time(NULL);
  hosts->hosts++;
  if(hosts->hosts>hosts->maxhosts)
   {
    hosts->maxhosts=hosts->hosts;
    hosts->maxhostsTS=time(NULL);
   }
  th=malloc(sizeof(TUH));
  strcpy(th->host, c);
  th->next=hosts->hashtable[hashuser(c)];
  th->prev=NULL;
  if(th->next) th->next->prev=th;
  hosts->hashtable[hashuser(c)]=th;
  th->count=1;
  nlist=malloc(sizeof(TNickList));
  nlist->prev=NULL;
  nlist->next=NULL;
  nlist->nick=user;
  th->nicklist=nlist;
  context;
  return th->count;
 }

void dellist(TBanList *list)
 {
  TBanList *tmplist;
  
  while(list)
   {
    tmplist=list;
    list=list->next;
    free(tmplist);
   }
 }

void delchannel(TChannel *channel)
 {
  TChannel *tmpchan;
  
  if(!channel) return;

  context;
  tmpchan=channel;
  if(tmpchan->next) tmpchan->next->prev=tmpchan->prev;
  if(tmpchan->prev) tmpchan->prev->next=tmpchan->next;
   else channels->hashtable[hashchannel(tmpchan->name)]=NULL;
  if(tmpchan->nicklist) logit("DELCHANNEL: Channel not empty. Please send bug report.\n");
  dellist(tmpchan->banlist);
  dellist(tmpchan->exclist);
  dellist(tmpchan->regexdeny);
  free(tmpchan);
  channels->avgchannels=calcavg(channels->avgchannels, channels->channels, channels->avgchannelsTS);
  channels->avgchannelsTS=time(NULL);
  channels->channels--;
 }

void delchanuser2(TChanL *clist, TUser *user)
 {
  TNickList *nlist;
  
  context;
  nlist=clist->channel->nicklist;
  while (nlist && (nlist->nick!=user)) nlist=nlist->next;
  if(!nlist)
   {
    logit("DELCHANUSER: No such nick %s on channel %s. Please send bug report.\n", user->nick, clist->channel->name);
    return;
   }
  context;
  if(nlist->next) nlist->next->prev=nlist->prev;
  if(nlist->prev) nlist->prev->next=nlist->next;
    else clist->channel->nicklist=nlist->next;
  if(clist->next) clist->next->prev=clist->prev;
  if(clist->prev) clist->prev->next=clist->next;
    else nlist->nick->chans=clist->next;
  clist->channel->users--;
  if(!clist->channel->users) delchannel(clist->channel);
  context;
  free(nlist);
  context;
  free(clist);
  context;
 }

void delchanuser(TChannel *channel, TUser *user)
 {
  TChanL *clist;
  
  context;
  clist=user->chans;
  while (clist && (clist->channel!=channel)) clist=clist->next;
  if(clist) delchanuser2(clist, user);
   else logit("DELCHANUSER: No such nick %s on channel %s.\n", user->nick, channel->name);
 }

void deluser(TUser *user)
 {
  TOper *oper;
  TChanL *chanl, *chanl2;
  TFOLst *fol, *foltmp;
#ifndef NO_PTHREAD  
  pthread_t pid;
#else
  int pid, exit_status;
#endif // NO_PTHREAD   
  char buff[1024];
  
  context;
  if(!user) return;
  for(fol=folist; fol; foltmp=fol, fol=fol->next)
   if(fol->user==user)
    {
     context;
     pid=fol->fuckover_thread;
     fol->stop=1;
#ifndef NO_PTHREAD
     pthread_join(pid, NULL);
#else
#ifdef WIN32
     TerminateThread(&pid, 0);
#else
     while(waitpid(fol->fuckover_thread, &exit_status, WNOHANG)< 1);
#endif // WIN32
#endif // NO_PTHREAD
     context;
     fol=foltmp;
    }
  oper=findcnoper(user->nick);
  if(oper) oper->lognick=NULL;

  sprintf(buff, "%s@%s", user->user, user->host);
//clone checking tu spravit
  deluserhost(buff, user);
  delhost(user->host, user);
  
  chanl=user->chans;
  context;
  while(chanl)
   {
    chanl2=chanl->next;
    delchanuser2(chanl, user);
    chanl=chanl2;
   }
  context;
  user->serv->avgusers=calcavg(user->serv->avgusers, user->serv->users, user->serv->avgusersTS);
  user->serv->avgusersTS=time(NULL);
  user->serv->users--;
  context;
  users->avgusers=calcavg(users->avgusers, users->users, users->avgusersTS);
  users->avgusersTS=time(NULL);
  users->users--;
  if(user->internal)
   {
    users->avglusers=calcavg(users->avglusers, users->lusers, users->avglusersTS);
    users->avglusersTS=time(NULL);
    users->lusers--;
   }
  if(user->internal==2) info.nickjupes--;
  if(user->invisible) users->invisible--;
  if(user->ircop) users->ircop--;
  context;
  if(user->next) user->next->prev=user->prev;
  if(user->prev) user->prev->next=user->next;
   else users->hashtable[hashuser(user->nick)]=user->next;
  free(user);
  context;
 }

void delserver(TServer *server)
 {
  TServer *serv, *serv2;
  TUser *user, *user2;
  int i;
  
  context;
  serv=servers;
  serv->temp=0;
  serv=serv->next;
  while(serv)
   {
    serv->temp=serv->uplink->temp||(server==serv);
    serv=serv->next;
   }
  context;
  for(i=0; i!=ESTUSERS; i++)
   {
    user=users->hashtable[i];
    while((user2=user))
    {
     user=user->next;
     if(user2->serv->temp) deluser(user2);
    }
   }
  context;
  serv=servers;
  serv2=NULL;
  while(serv)
   {
    if(serv->temp)
     {
      if(serv2) serv2->next=serv->next;
       else servers=serv->next;
      if(serv->internal==2) info.servjupes--;
      free(serv);
      info.avgservers=calcavg(info.avgservers, info.servers, info.avgserversTS);
      info.avgserversTS=time(NULL);
      info.servers--;
      serv=serv2->next;
     } else
     {
      serv2=serv;
      serv=serv->next;
     }
   }
 }

void adduser(char *nick, char *user, char *host, char *server, time_t ts, int hops, int ircop, int invisible, int internal, char *name, ...)
 {
  TUser *tmpuser;
  va_list va;
  char buff[1024];

  context;
  if(finduser(nick))
   {
    logit("ADDUSER: Nick %s already exists.\n", nick);
    return;
   }
   
  tmpuser=malloc(sizeof(TUser));

  sprintf(buff, "%s@%s", user, host);
//clone checking tu spravit
  adduserhost(buff, tmpuser);
  addhost(host, tmpuser);
  info.joins++;
  users->avgusers=calcavg(users->avgusers, users->users, users->avgusersTS);
  users->avgusersTS=time(NULL);
  users->users++;
  if(internal==2) info.nickjupes++;
  if(invisible) users->invisible++;
  if(ircop) users->ircop++;
  if(users->users>users->maxusers)
   {
    users->maxusers=users->users;
    users->maxusersTS=time(NULL);
   }
  context;
  if(internal)
   {
    users->avglusers=calcavg(users->avglusers, users->lusers, users->avglusersTS);
    users->avglusersTS=time(NULL);
    users->lusers++;
    if(users->lusers>users->maxlusers)
     {
      users->maxlusers=users->lusers;
      users->maxlusersTS=time(NULL);
     }
   }

  va_start(va, name);
  vsnprintf(buff, 1024, name, va);
  va_end(va);
  
  context;
  tmpuser->next=users->hashtable[hashuser(nick)];
  if(tmpuser->next) tmpuser->next->prev=tmpuser;
  tmpuser->prev=NULL;
  users->hashtable[hashuser(nick)]=tmpuser;
  tmpuser->chans=NULL;
  strcpy(tmpuser->user, user);
  strcpy(tmpuser->host, host);
  strcpy(tmpuser->name, buff);
  strcpy(tmpuser->nick, nick);
  tmpuser->ts=ts;
  context;
  tmpuser->ircop=ircop;
  tmpuser->invisible=invisible;
  tmpuser->hops=hops;
  tmpuser->internal=internal;
  if(!(tmpuser->serv=findserver(server)))
   {
    logit("ADDUSER: No such server %s.", server);
    fatal(2);
   }

  tmpuser->serv->avgusers=calcavg(tmpuser->serv->avgusers, tmpuser->serv->users, tmpuser->serv->avgusersTS);
  tmpuser->serv->avgusersTS=time(NULL);
  tmpuser->serv->users++;
  context;
  if(tmpuser->serv->users>tmpuser->serv->maxusers)
   {
    tmpuser->serv->maxusers=tmpuser->serv->users;
    tmpuser->serv->maxusersTS=time(NULL);
   }
 }

TChannel *addchannel(char *name, int ts)
 {
  TChannel *tmpch;

  context;
  if(findchannel(name))
   {
    logit("ADDCHANNEL: Channel %s already exists.\n", name);
    return NULL;
   }
  
  channels->avgchannels=calcavg(channels->avgchannels, channels->channels, channels->avgchannelsTS);
  channels->avgchannelsTS=time(NULL);
  channels->channels++;
  if(channels->channels>channels->maxchannels)
   {
    channels->maxchannels=channels->channels;
    channels->maxchannelsTS=time(NULL);
   }
  context;

  tmpch=malloc(sizeof(TChannel));
  tmpch->next=channels->hashtable[hashchannel(name)];
  if(tmpch->next) tmpch->next->prev=tmpch;
  tmpch->prev=NULL;
  channels->hashtable[hashchannel(name)]=tmpch;

  strcpy(tmpch->name, name);
  tmpch->users=0;
  tmpch->topic[0]=0;
  tmpch->key[0]=0;
  tmpch->inviteo=tmpch->userlimit=tmpch->moderated=tmpch->nomsg=tmpch->private=tmpch->secure=tmpch->topicopo=0;
  tmpch->ts=ts;
  tmpch->nicklist=NULL;
  tmpch->banlist=NULL;
  tmpch->exclist=NULL;
  tmpch->regexdeny=NULL;
  context;
  return tmpch;
 }

TBanList *addban(TChannel *ch, char *ban, TBanList *banlist)
 {
  TBanList *blist;
  
  context;
  blist=malloc(sizeof(TBanList));
  strcpy(blist->ban, ban);
  blist->prev=NULL;
  blist->next=banlist;
  if(banlist) blist->next->prev=blist;
  context;
  return blist;
 }

TBanList *delban(TChannel *ch, char *ban, TBanList *banlist)
 {
  TBanList *blist;
  
  context;
  blist=banlist;
  while(blist && strcasecmp(ban, blist->ban)) blist=blist->next;
  if(!blist) return banlist;
  if(blist->next) blist->next->prev=blist->prev;
  if(blist->prev) blist->prev->next=blist->next; else
   {
    free(blist);
    return blist->next;
   }
  free(blist);
  return banlist;
 }

int setmode(TChannel *ch, char *mode, int argb)
 {
  TNickList *nlist;
  int plus, i;
  
  context;
  plus=1;
  i=0;
  while(mode[i])
   {
    switch(mode[i])
     {
      case '+': case '-':
       {
        plus=mode[i]=='+';
        break;
       }
      case 'b':
       {
        if(plus) ch->banlist=addban(ch, arg[argb++], ch->banlist); else
                 ch->banlist=delban(ch, arg[argb++], ch->banlist);
        break;
       }
      case 'i':
       {
        ch->inviteo=plus;
        break;
       }
      case 'k':
       {
        if(plus) strcpy(ch->key, arg[argb++]); else
         {
          ch->key[0]=0;
          argb++;
         }
        break;
       }
      case 'l':
       {
        if(plus) ch->userlimit=atoi(arg[argb++]); else ch->userlimit=0;
        break;
       }
      case 'm':
       {
        ch->moderated=plus;
        break;
       }
      case 'n':
       {
        ch->nomsg=plus;
        break;
       }
      case 'o':
       {
        for(nlist=ch->nicklist; nlist; nlist=nlist->next)
         if(!strcmp(nlist->nick->nick, arg[argb]))
          nlist->op=plus;
        argb++;
        break;
       }
      case 'p':
       {
        ch->private=plus;
        break;
       }
      case 's':
       {
        ch->secure=plus;
        break;
       }
      case 't':
       {
        ch->topicopo=plus;
        break;
       }
      case 'v':
       {
        for(nlist=ch->nicklist; nlist; nlist=nlist->next)
         if(!strcmp(nlist->nick->nick, arg[argb]))
          nlist->vop=plus;
        argb++;
        break;
       }
      case 'e':
       {
        if(plus) ch->exclist=addban(ch, arg[argb++], ch->exclist); else
                 ch->exclist=delban(ch, arg[argb++], ch->exclist);
        break;
       }
      case 'd':
       {
        if(plus) ch->regexdeny=addban(ch, arg[argb++], ch->regexdeny); else
                 ch->regexdeny=delban(ch, arg[argb++], ch->regexdeny);
        break;
       }
     }
    i++;
   }
  context;
  return argb;
 }

void addchanuser(TChannel *channel, TUser *user, int op, int vop)
 {
  TNickList *nlist;
  TChanL *clist;
  
  context;
  if(!channel || !user)
   {
    logit("ADDCHANUSER: No such nick/channel.\n");
    fatal(2);
   }
  nlist=malloc(sizeof(TNickList));
  nlist->prev=NULL;
  nlist->next=channel->nicklist;
  if(nlist->next) nlist->next->prev=nlist;
  nlist->op=op;
  nlist->vop=vop;
  nlist->nick=user;
  channel->users++;
  channel->nicklist=nlist;
  
  clist=malloc(sizeof(TChanL));
  clist->prev=NULL;
  clist->next=user->chans;
  if(clist->next) clist->next->prev=clist;
  clist->channel=channel;
  context;
  user->chans=clist;
 }

void addserver(char *name, char *desc, int hops, int internal, char *uplink)
 {
  TServer *server;
  
  context;
  if(servers)
   {
    server=servers;
    while(server->next)
     {
      if(match(name, server->name) || match(server->name, name))
       {
        context;
        logit("ADDSERVER: Server %s already exists (%s).\n", name, server->name);
        return;
       }
      server=server->next;
     }
    if(match(name, server->name) || match(server->name, name))
     {
      logit("ADDSERVER: Server %s already exists (%s).\n", name, server->name);
      return;
     }
    server->next=malloc(sizeof(TServer));
    server=server->next;
   } else server=servers=malloc(sizeof(TServer));
  context;
  info.avgservers=calcavg(info.avgservers, info.servers, info.avgserversTS);
  info.avgserversTS=time(NULL);
  info.servers++;
  if(internal==2) info.servjupes++;
  if(info.servers>info.maxservers)
   {
    info.maxservers=info.servers;
    info.maxserversTS=time(NULL);
   }
  server->next=NULL;
  strcpy(server->name, name);
  strcpy(server->desc, desc);
  context;
  server->ts=time(NULL);
  server->hops=hops;
  server->users=0;
  server->maxusers=0;
  server->avgusers=0;
  server->maxusersTS=0;
  server->avgusersTS=0;
  server->internal=internal;
  context;
  server->uplink=findserver(uplink);
 }
